import 'package:intl/intl.dart';

import '../utils/hijri_date.dart';

/// Jakim E-solat API model.
/// Generated by https://jsontodart.zariman.dev/
class JakimEsolatModel {
  List<PrayerTime>? prayerTime;
  String? status;
  String? serverTime;
  // String? periodType;
  // String? lang;
  String? zone;
  // String? bearing;

  JakimEsolatModel({
    this.prayerTime,
    this.status,
    this.serverTime,
    // this.periodType,
    // this.lang,
    this.zone,
    // this.bearing
  });

  JakimEsolatModel.fromJson(Map<String, dynamic> json) {
    prayerTime = json["prayerTime"] == null
        ? null
        : (json["prayerTime"] as List)
            .map((e) => PrayerTime.fromJson(e))
            .toList();
    status = json["status"];
    serverTime = json["serverTime"];
    // periodType = json["periodType"];
    // lang = json["lang"];
    zone = json["zone"];
    // bearing = json["bearing"];
  }
}

class PrayerTime {
  late HijriDate hijri;
  late DateTime imsak;
  late DateTime fajr;
  late DateTime syuruk;
  late DateTime dhuha;
  late DateTime dhuhr;
  late DateTime asr;
  late DateTime maghrib;
  late DateTime isha;
  PrayerTime(
      {required this.hijri,
      required this.imsak,
      required this.fajr,
      required this.syuruk,
      required this.dhuhr,
      required this.asr,
      required this.maghrib,
      required this.isha});

  PrayerTime.fromJson(Map<String, dynamic> json) {
    hijri = HijriDate.fromJson(json["hijri"]);
    var date = json["date"];
    var imsakRaw = json["imsak"];
    var fajrRaw = json["fajr"];
    var syurukRaw = json["syuruk"];
    var dhuhrRaw = json["dhuhr"];
    var asrRaw = json["asr"];
    var maghribRaw = json["maghrib"];
    var ishaRaw = json["isha"];

    var monthMap = [
      'Jan',
      'Feb',
      'Mar',
      'Apr',
      'May',
      'Jun',
      'Jul',
      'Aug',
      'Sep',
      'Oct',
      'Nov',
      'Dec'
    ];

    String pattern = 'd-M-y HH:mm:ss';

    // Changing Text month to its numeric equivalent
    // eg: 2-Okt-2021 will be 2-10-2021

    var monthNumeric =
        monthMap.indexWhere((element) => date.contains(element)) + 1;

    date = date.replaceRange(3, 6, monthNumeric.toString());

    imsak = DateFormat(pattern).parse('$date $imsakRaw');
    fajr = DateFormat(pattern).parse('$date $fajrRaw');
    syuruk = DateFormat(pattern).parse('$date $syurukRaw');
    dhuha = syuruk.add(const Duration(minutes: 28));
    dhuhr = DateFormat(pattern).parse('$date $dhuhrRaw');
    asr = DateFormat(pattern).parse('$date $asrRaw');
    maghrib = DateFormat(pattern).parse('$date $maghribRaw');
    isha = DateFormat(pattern).parse('$date $ishaRaw');
  }

  @override
  String toString() {
    return "{hijri: $hijri, fajr: $fajr, dhuhr: $dhuhr, asr: $asr, maghrib: $maghrib, isha: $isha}";
  }
}
